#!/bin/sh
# Code copyright (c) 2006 Axentra Corporation.  All rights reserved.

# Main starts here
PATH=$PATH:/sbin:/usr/sbin:/bin:/usr/bin

. /root/functions.sh
get_platform
set_platform_param $PLATFORM

if `cat /proc/mounts | grep -q '^/dev/md0 '`; then
    PARTITION_ROOT=/dev/md0
    PARTITION_SWAP=/dev/md1
    PARTITION_HOME=/dev/md2
else
    PARTITION_ROOT=/dev/sda1
    PARTITION_SWAP=/dev/sda2
    PARTITION_HOME=/dev/sda3
fi

if [ x$1 = x-chroot ]; then
    echo "We are now in chrooted /mnt."
    
    /etc/init.d/cert start

    [ -f /var/log/messages ] && echo > /var/log/messages
    [ -f /var/log/httpd/error_log ] && echo > /var/log/httpd/error_log
    
    if [ -f /boot/update_ondisk_kernel ]; then
        case `uname -r` in 
                *6281)
                    dd if=/boot/uImage.6281 of=/dev/sda6
                    ;;
                *6192)
                    dd if=/boot/uImage.6192 of=/dev/sda6
                    ;;
                *6281A)
                    dd if=/boot/uImage.6281a of=/dev/sda6
                    ;;
                *6192a)
                    dd if=/boot/uImage.6192a of=/dev/sda6
                    ;;
                *motorola)
                    dd if=/boot/uImage.motorola of=/dev/sda6
                    ;;
                *sheeva)
                    dd if=/boot/uImage.sheeva of=/dev/sda6
                    # or the flash related stuff
                    ;;
                *ispace)
                    dd if=/boot/uImage.ispace of=/dev/sda6
                    ;;
                2.6.22.7)
                    dd if=/boot/uImage of=/dev/sda6
                    ;;
                *)
                    dd if=/boot/uImage of=/dev/sda6     
                    ;;
          esac 
    fi
    if [ ! -f /var/www/.ssh/id_dsa ]; then 
        /usr/bin/ssh-keygen -q -t dsa -f /var/www/.ssh/id_dsa -C '' -b 1024 -P ''
        /bin/chown apache.apache /var/www/.ssh/id_dsa*
        /bin/chmod 600 /var/www/.ssh/id_dsa*
    fi
    echo "About to exit from chrooted environment."
    exit
fi


AXENTRA_TAG=AX_USB_HD_QUERY_RESULT

PATH=$PATH:/sbin:/usr/sbin:/bin:/usr/bin

if [ $PLATFORM = NSLU2 ]; then
REAL_MAC=`/usr/bin/strings /dev/mtdblock1 | /bin/sed -n 's/hw_addr=//p'`
/bin/sed -e "s/hw_addr=.*$/hw_addr=${REAL_MAC}/" -i /mnt/etc/default/sysconf
sed -e "s/^\([^#].*ether \)..:..:..:..:..:..$/\1${REAL_MAC}/" -i /mnt/etc/network/interfaces

cat >/mnt/etc/sysconfig/network-scripts/ifcfg-eth0 <<EOF
DEVICE=eth0
BOOTPROTO=dhcp
ONBOOT=yes
MACADDR=$REAL_MAC
EOF

cat > /boot/axentra_preboot <<EOF
#!/bin/sh
#Rename OpenSlug recovery flag to Redhat flag.
test -f /mnt/.recovery && mv /mnt/.recovery /mnt/.autofsck
rm -f /var/run/confirm
EOF
chmod +x /boot/axentra_preboot
sync

#squeeze it into firmware boot process
grep -q axentra_preboot /boot/disk || sed -i '/# checkmount checks for sh,/a\
        \/boot\/axentra_preboot' /boot/disk
sync
else    #NSLU2
    cat > /mnt/etc/fstab <<EOF
# This file is edited by fstab-sync - see 'man fstab-sync' for details
#
# /etc/fstab: static file system information.
#
# file system   mount       type    options           dump    pass
${PARTITION_ROOT}       /           ext3    defaults          1       1
proc            /proc       proc    defaults          0       0
devpts          /dev/pts    devpts  gid=5,mode=620    0       0
sysfs           /sys        sysfs   defaults          0       0
${PARTITION_HOME}   /home   ext3    defaults,bsdgroups,quota,noatime,nodiratime    0   0
## other examples, uncomment to activate
${PARTITION_SWAP}      swap        swap    defaults          0       0
EOF
fi              #NSLU2

#Supposing previous steps leave /dev/sda1 mounted at /mnt
cp $0 /mnt/root/
chmod +x /mnt/root/$0

#Copy functions.sh to /mnt/root since it is needed
cp /root/functions.sh /mnt/root/
chmod +x /mnt/root/functions.sh

echo "We will chroot into /mnt, and run part of script there."
chroot /mnt /root/$0 -chroot

echo "Returned from chroot environment."
#Be ugly now, scripts run in chroot won't have /proc/mounts.
#So run chroot scripts will overwrite $PARTITION_* variables with wrong values.
if `cat /proc/mounts | grep -q '^/dev/md0 '`; then
    PARTITION_ROOT=/dev/md0
    PARTITION_SWAP=/dev/md1
    PARTITION_HOME=/dev/md2

    DISKS=`cat /proc/partitions | awk '{if (/sd[abcd]$/) printf $4 " "}'`
    DISK_NUMBER=`echo $DISKS | wc -w | awk '{print $1}'`

if [ $DISK_NUMBER -eq 2 ]; then
    cat > /mnt/etc/raidtab <<EOF
# raid-0 configuration
raiddev			/dev/md0
    raid-level		0 
    persistent-superblock   0    
    chunk-size		16
    nr-raid-disks		2
    nr-spare-disks		0
    device			/dev/sda1
    raid-disk		0
    device			/dev/sdb1
    raid-disk		1
raiddev			/dev/md1
    raid-level		0 
    persistent-superblock   0    
    chunk-size		16
    nr-raid-disks		2
    nr-spare-disks		0
    device			/dev/sda2
    raid-disk		0
    device			/dev/sdb2
    raid-disk		1
raiddev			/dev/md2
    raid-level		0 
    persistent-superblock   0    
    chunk-size		16
    nr-raid-disks		2
    nr-spare-disks		0
    device			/dev/sda3
    raid-disk		0
    device			/dev/sdb3
    raid-disk		1
EOF
else if [ $DISK_NUMBER -eq 3 ]; then
    cat > /mnt/etc/raidtab <<EOF
# raid-0 configuration
raiddev			/dev/md0
    raid-level		5 
    persistent-superblock   0    
    chunk-size		16
    nr-raid-disks		3
    nr-spare-disks		0
    device			/dev/sda1
    raid-disk		0
    device			/dev/sdb1
    raid-disk		1
    device			/dev/sdc1
    raid-disk		2
raiddev			/dev/md1
    raid-level		5 
    persistent-superblock   0    
    chunk-size		16
    nr-raid-disks		3
    nr-spare-disks		0
    device			/dev/sda2
    raid-disk		0
    device			/dev/sdb2
    raid-disk		1
    device			/dev/sdc2
    raid-disk		2
raiddev			/dev/md2
    raid-level		5 
    persistent-superblock   0    
    chunk-size		16
    nr-raid-disks		3
    nr-spare-disks		0
    device			/dev/sda3
    raid-disk		0
    device			/dev/sdb3
    raid-disk		1
    device			/dev/sdc3
    raid-disk		2
EOF
else if [ $DISK_NUMBER -eq 4 ]; then
    cat > /mnt/etc/raidtab <<EOF
# raid-0 configuration
raiddev			/dev/md0
    raid-level		5 
    persistent-superblock   0    
    chunk-size		16
    nr-raid-disks		4
    nr-spare-disks		0
    device			/dev/sda1
    raid-disk		0
    device			/dev/sdb1
    raid-disk		1
    device			/dev/sdc1
    raid-disk		2
    device			/dev/sdd1
    raid-disk		3
raiddev			/dev/md1
    raid-level		5 
    persistent-superblock   0    
    chunk-size		16
    nr-raid-disks		4
    nr-spare-disks		0
    device			/dev/sda2
    raid-disk		0
    device			/dev/sdb2
    raid-disk		1
    device			/dev/sdc2
    raid-disk		2
    device			/dev/sdd2
    raid-disk		3
raiddev			/dev/md2
    raid-level		5 
    persistent-superblock   0    
    chunk-size		16
    nr-raid-disks		4
    nr-spare-disks		0
    device			/dev/sda3
    raid-disk		0
    device			/dev/sdb3
    raid-disk		1
    device			/dev/sdc3
    raid-disk		2
    device			/dev/sdd3
    raid-disk		3
EOF
fi
fi
fi
else
    PARTITION_ROOT=/dev/sda1
    PARTITION_SWAP=/dev/sda2
    PARTITION_HOME=/dev/sda3
    rm -f /mnt/var/www/admin/raid.pm
fi

#unmount /mnt/home first
/bin/umount /mnt/home

test -f /mnt/axentra_reinstall && rm -f /mnt/axentra_reinstall
if `/bin/umount /mnt` ;
then
        /bin/echo "${AXENTRA_TAG}: state=DISMOUNTED"
else
        /bin/echo "${AXENTRA_TAG}: state=DISMOUNT_FAILED"
        /bin/echo "${AXENTRA_TAG}: action=quit"
        exit
fi

if [ $PLATFORM = NSLU2 ]; then
/usr/sbin/turnup disk /dev/sda1 -t ext3
/bin/echo "${AXENTRA_TAG}: state=TURNUP_FINISHED"
fi

/bin/echo "${AXENTRA_TAG}: action=end"
