#!/bin/sh
# Code copyright (c) 2006 Axentra Corporation.  All rights reserved.

#Common functions for Hipserv install process

SDA7FS=`parted -s /dev/sda print | grep ^7 | awk '{print $5}'`
if [ "$SDA7FS" = "ext3" ]; then
    BOOTS_FROM_DISK=1
    NEED_EXTENDED=1
    ROOTFS_SIZE=20
    KERNEL_SIZE=5
    SPARE_SIZE=20
fi

get_platform() {
    if [ x$PLATFORM = x ]; then         #We have to detect PLATFORM
        MACH=`uname -m`           # -m : machine type
    else 
        return
    fi

    case "$MACH" in
        armv5teb)
            PLATFORM=NSLU2 
            ;;
        armv5tejl)
            PLATFORM=MARVELL
            ;;
        ppc)
            PLATFORM=USI8347
            ;;
        *)
            echo "\$PLATFORM=$PLATFORM, unknown environment variable, aborted."
            exit -1
            ;;
    esac
}

set_platform_param() {
  case "$1" in
    NSLU2)
           PARTITION_DEV=/dev/sda
           PARTED_DIR=/tmp
           ;;
    MARVELL)
           PARTITION_DEV=/dev/sda
           PARTED_DIR=/sbin
           SWAPSIZE=512
           ;;
    USI8347)
           PARTITION_DEV=/dev/sda
           PARTED_DIR=/sbin
           SWAPSIZE=512
           ;;
    usi)
           ;;
    *)
        echo "\$PLATFORM=$PLATFORM, unknown environment variable, aborted."
        exit -1
        ;;
  esac

  PARTITION_ROOT=${PARTITION_DEV}1
  PARTITION_SWAP=${PARTITION_DEV}2
  PARTITION_HOME=${PARTITION_DEV}3

}

disk_partition() {
    local HDSTART HDEND HDSIZE PARTITION_DEV \
        HDROOTEND HDSWAPSTART HDSWAPEND HDHOMESTART HDHOMEEND HDFLASHSTART \
        HDKERNELSTART HDROOTFSSTART

    PARTITION_DEV=$1
    IS_RAID=$2

    if [ x"$BOOTS_FROM_DISK" != x1 -o "$PARTITION_DEV" != "/dev/sda" ]; then
        #explicitly remove partitions 1-3 in case mklabel is not called to wipe them out.
        $PARTED_DIR/parted -s $PARTITION_DEV rm 3
        $PARTED_DIR/parted -s $PARTITION_DEV rm 2
        $PARTED_DIR/parted -s $PARTITION_DEV rm 1
        $PARTED_DIR/parted -s $PARTITION_DEV rm 4
    fi

    /bin/echo "${AXENTRA_TAG}: action=message"
    $PARTED_DIR/parted -s $PARTITION_DEV print || (echo "Recreating disk label on $PARTITION_DEV.."; $PARTED_DIR/parted -s $PARTITION_DEV mklabel msdos; $PARTED_DIR/parted -s $PARTITION_DEV print)
    
    # Watch out, we trimmed the fractions!
    HDSTART=`$PARTED_DIR/parted -s $PARTITION_DEV print | awk -F ': |-' '/Disk geometry for/{print $2}' | awk -F '.' '{print $1}'`
    HDEND=`$PARTED_DIR/parted -s $PARTITION_DEV print | awk -F '[ :-]' '/Disk geometry for/{print $7,$9}' | awk -F '.' '{print $1}'`
    HDSIZE=`expr $HDEND - $HDSTART`
    #echo "start=$HDSTART"
    #echo "end=$HDEND"
    #echo "size=$HDSIZE"
    /bin/echo "${AXENTRA_TAG}: disk_size=${HDSIZE}Meg"
    
    if [ $HDSIZE -lt $HDMINSIZE ];
    then
            /bin/echo "${AXENTRA_TAG}: action=quit"
            exit
    fi
    
    #Just to be save for now
    HDROOTEND=$ROOTMINSIZE
    HDSWAPSTART=$HDROOTEND
    HDSWAPEND=`expr $HDSWAPSTART + $SWAPSIZE`
    HDHOMESTART=$HDSWAPEND
    if [ "$NEED_EXTENDED" ];then 
        HDHOMEEND=`expr $HDEND - $SPARE_SIZE - $ROOTFS_SIZE - $KERNEL_SIZE`
        HDFLASHSTART=$HDHOMEEND
        HDKERNELSTART=`expr $HDEND - $ROOTFS_SIZE - $KERNEL_SIZE`
        HDROOTFSSTART=`expr $HDEND - $ROOTFS_SIZE`
    else
        HDHOMEEND=$HDEND
    fi
    
    if [ x"$BOOTS_FROM_DISK" != x1 -o "$PARTITION_DEV" != "/dev/sda" ]; then
        $PARTED_DIR/parted -s $PARTITION_DEV mkpart p 0 $HDROOTEND
        $PARTED_DIR/parted -s $PARTITION_DEV mkpart p linux-swap $HDSWAPSTART $HDSWAPEND
        $PARTED_DIR/parted -s $PARTITION_DEV mkpart p $HDHOMESTART $HDHOMEEND
        $PARTED_DIR/parted -s $PARTITION_DEV set 1 boot on
        if [ $IS_RAID = "yes" ]; then
            $PARTED_DIR/parted -s $PARTITION_DEV set 1 raid on
            $PARTED_DIR/parted -s $PARTITION_DEV set 2 raid on
            $PARTED_DIR/parted -s $PARTITION_DEV set 3 raid on
        fi
        if [ "$NEED_EXTENDED" ];then 
            $PARTED_DIR/parted -s $PARTITION_DEV mkpart e $HDFLASHSTART $HDEND
            $PARTED_DIR/parted -s $PARTITION_DEV mkpart l $HDFLASHSTART $HDKERNELSTART
            $PARTED_DIR/parted -s $PARTITION_DEV mkpart l $HDKERNELSTART $HDROOTFSSTART
            $PARTED_DIR/parted -s $PARTITION_DEV mkpart l $HDROOTFSSTART $HDEND
        fi
    fi
}

has_raid() {
    PARTITION_DEV=$1
    NUMBER_OF_DISKS=$2
    RAID_TYPE=$3
    HAS_RAID_ON_MD0=`mdadm --query ${PARTITION_DEV}0 | grep "$RAID_TYPE $NUMBER_OF_DISKS devices"`
    HAS_RAID_ON_MD1=`mdadm --query ${PARTITION_DEV}1 | grep "$RAID_TYPE $NUMBER_OF_DISKS devices"`
    HAS_RAID_ON_MD2=`mdadm --query ${PARTITION_DEV}2 | grep "$RAID_TYPE $NUMBER_OF_DISKS devices"`
}
