#!/bin/sh
# Code copyright (c) 2006 Axentra Corporation.  All rights reserved.

# Main starts here
PATH=$PATH:/sbin:/usr/sbin:/bin:/usr/bin

set -x

. /root/functions.sh

ROOTMINSIZE=3000
HOMEMINSIZE=500
SWAPSIZE=128

get_platform
set_platform_param $PLATFORM
if [ -z $PLATFORM ]; then
    echo 'A environment variable $PLATFORM is needed for running this program.'
    exit -1
fi

SERVER=10.0.0.2
MOUNTDIR=/mnt

case "$PLATFORM" in
    iomega8241)
           if [ "$1" ]; then
               PARTITION_DEV=$1
           else
               PARTITION_DEV=/dev/hda 
           fi
           PARTED_DIR=/sbin
           MOUNTDIR=/mnt/hipserv
           ;;
    riogrande)
           if [ "$1" ]; then
               PARTITION_DEV=$1
           else
               PARTITION_DEV=/dev/sda 
           fi
           PARTED_DIR=/sbin
           MOUNTDIR=/mnt/hipserv
           ;;
    nslu2)
           if [ "$1" ]; then
               PARTITION_DEV=$1
           else
               PARTITION_DEV=/dev/sda 
           fi
           PARTED_DIR=/tmp
           ;;
    marvell|lacie)
           if [ "$1" ]; then
               PARTITION_DEV=$1
           else
               PARTITION_DEV=/dev/sda            
           fi
           PARTED_DIR=/sbin
           MOUNTDIR=/mnt/hipserv
           KERNEL_FILE=uImage.$PLATFORM
           MINI_ROOTFS=mini-rootfs.marvell
           ROOTFS_SIZE=20
           KERNEL_SIZE=5
           SPARE_SIZE=475
           NEED_EXTENDED=1
           SERVER_ARCHIVE_DIR=data/marvell/archives
           SWAPSIZE=512
           ;;
    usi)
           if [ "$1" ]; then
               PARTITION_DEV=$1
           else
               PARTITION_DEV=/dev/sda            
           fi
           PARTED_DIR=/sbin
           MOUNTDIR=/mnt/hipserv
           MINI_ROOTFS=mini-rootfs.marvell
           ROOTFS_SIZE=20
           KERNEL_SIZE=8 #minimum partition size is 8MB
           SPARE_SIZE=8  #minimum partition size is 8MB
           NEED_EXTENDED=1
           SERVER_ARCHIVE_DIR=data/marvell/archives
           ;;
    usi8347)
           if [ "$1" ]; then
               PARTITION_DEV=$1
           else
               PARTITION_DEV=/dev/sda 
           fi
           PARTED_DIR=/sbin
           MOUNTDIR=/mnt/hipserv
           SWAPSIZE=512
           ;;
    NSLU2|MARVELL|USI8347)
            #Symbols for CD Installer
            ;;
    *)          
        if [ "$1" ]; then 
            PARTITION_DEV=$1
            PARTED_DIR=/sbin
            MOUNTDIR=/mnt/hipserv
        else
            echo "\$PLATFORM=$PLATFORM, unknown environment variable, aborted."
            exit -1
        fi
        ;;
esac

PARTITION_ROOT=${PARTITION_DEV}1            
PARTITION_SWAP=${PARTITION_DEV}2            
PARTITION_HOME=${PARTITION_DEV}3
if [ "$MINI_ROOTFS" ]; then
    PARTITION_ROOTFS=${PARTITION_DEV}7
fi
if [ "$KERNEL_FILE" ]; then
    PARTITION_KERNEL=${PARTITION_DEV}6
fi

AXENTRA_TAG=AX_USB_HD_QUERY_RESULT
HDMINSIZE=`expr $ROOTMINSIZE + $HOMEMINSIZE + $SWAPSIZE`

#in case the program just scp copied does not have execution permission.
/bin/chmod +x $PARTED_DIR/parted

DISKS=`cat /proc/partitions | awk '{if (/sd[abcd]$/) printf $4 " "}'`
DISK_NUMBER=`echo $DISKS | wc -w | awk '{print $1}'`

#Stop active RAID if there happen to be, as always happens in tests.
case "$DISK_NUMBER" in
    0)
        echo "We have no drive."
        echo "${AXENTRA_TAG}: action=quit"
        exit
        ;;
    1) 
        echo "We have one drive."
        disk_partition $PARTITION_DEV no
        ;;
    2) 
        echo "We have two drives."
        mdadm --stop /dev/md0
        mdadm --stop /dev/md1
        mdadm --stop /dev/md2
        for t in `echo $DISKS`
        do
            disk_partition /dev/$t yes
        done
        PARTITION_DEV=/dev/md
        PARTITION_ROOT=${PARTITION_DEV}0            
        PARTITION_SWAP=${PARTITION_DEV}1            
        PARTITION_HOME=${PARTITION_DEV}2
        mdadm -v -C $PARTITION_ROOT --level=1 --raid-devices=$DISK_NUMBER -f -R /dev/sd[ab]1
        mdadm -v -C $PARTITION_SWAP --level=1 --raid-devices=$DISK_NUMBER -f -R /dev/sd[ab]2
        mdadm -v -C $PARTITION_HOME --level=1 --raid-devices=$DISK_NUMBER -f -R /dev/sd[ab]3
        #Note, we don't make arrays on miniroot and kernel partitions.
        ;;
    3) 
        mdadm --stop /dev/md0
        mdadm --stop /dev/md1
        mdadm --stop /dev/md2
        echo "We have three drives."
        for t in `echo $DISKS`
        do
            disk_partition /dev/$t yes
        done
        PARTITION_DEV=/dev/md
        PARTITION_ROOT=${PARTITION_DEV}0            
        PARTITION_SWAP=${PARTITION_DEV}1            
        PARTITION_HOME=${PARTITION_DEV}2
        mdadm -v -C $PARTITION_ROOT --level=5 --raid-devices=$DISK_NUMBER -f -R /dev/sd[abc]1
        mdadm -v -C $PARTITION_SWAP --level=5 --raid-devices=$DISK_NUMBER -f -R /dev/sd[abc]2
        mdadm -v -C $PARTITION_HOME --level=5 --raid-devices=$DISK_NUMBER -f -R /dev/sd[abc]3
        ;;
    4) 
        echo "We have four drives."
        for t in `echo $DISKS`
        do
            disk_partition /dev/$t yes
        done
        PARTITION_DEV=/dev/md
        PARTITION_ROOT=${PARTITION_DEV}0            
        PARTITION_SWAP=${PARTITION_DEV}1            
        PARTITION_HOME=${PARTITION_DEV}2
        has_raid ${PARTITION_DEV} 4 raid5
        if [ ! "$HAS_RAID_ON_MD0" ]; then
            mdadm --stop /dev/md0
            mdadm -v -C $PARTITION_ROOT --level=5 --raid-devices=$DISK_NUMBER -f -R /dev/sd[abcd]1
        fi
        if [ ! "$HAS_RAID_ON_MD1" ]; then
            mdadm --stop /dev/md1
            mdadm -v -C $PARTITION_SWAP --level=5 --raid-devices=$DISK_NUMBER -f -R /dev/sd[abcd]2
        fi
        if [ ! "$HAS_RAID_ON_MD2" ]; then
            mdadm --stop /dev/md2
            mdadm -v -C $PARTITION_HOME --level=5 --raid-devices=$DISK_NUMBER -f -R /dev/sd[abcd]3
        fi
        ;;
esac

#Bigger one should be first since the progress indicator
#watches the first one
/sbin/mkswap $PARTITION_SWAP
/sbin/swapon $PARTITION_SWAP
/sbin/mkfs.ext3 $PARTITION_HOME
/sbin/mkfs.ext3 $PARTITION_ROOT
#/sbin/mkswap $PARTITION_SWAP
if [ "$MINI_ROOTFS" ]; then
    /sbin/mkfs.ext3 $PARTITION_ROOTFS
fi

#For development unit, turnup disk and reboot
#turnup disk -i /dev/sda1 -t ext3

#If mount failed, extracting tar ball will flood $MOUNTDIR
if `/bin/mount -t ext3 $PARTITION_ROOT $MOUNTDIR` ;
then
    #mount /home
    test -d $MOUNTDIR/home || mkdir $MOUNTDIR/home
    if `mount -t ext3 $PARTITION_HOME $MOUNTDIR/home` ; then
        echo "${AXENTRA_TAG}: state=MOUNTED"
    else
        echo "${AXENTRA_TAG}: state=MOUNT_FAILED"
        echo "${AXENTRA_TAG}: action=quit"
        exit
    fi
else
    /bin/echo "${AXENTRA_TAG}: state=MOUNT_FAILED"
    /bin/echo "${AXENTRA_TAG}: action=quit"
    exit
fi

#Install the mini rootfs
if [ "$MINI_ROOTFS" ]; then
    mkdir -p /mnt/rootfs
    /bin/mount -t ext3 $PARTITION_ROOTFS /mnt/rootfs
    rm -f $MINI_ROOTFS*
    wget ftp://$SERVER/$SERVER_ARCHIVE_DIR/$MINI_ROOTFS* ||
        (echo "wget failed, in fetching $MINI_ROOTFS" ; kill $$)

    tar xzf $MINI_ROOTFS* -C /mnt/rootfs
    umount /mnt/rootfs
fi

#Install the kernel
if [ "$KERNEL_FILE" ]; then
    rm -f $KERNEL_FILE
    wget ftp://$SERVER/$SERVER_ARCHIVE_DIR/$KERNEL_FILE ||
        (echo "wget failed, in fetching $KERNEL_FILE" ; kill $$)

    dd if=$KERNEL_FILE of=$PARTITION_KERNEL bs=20b
fi


/bin/echo "${AXENTRA_TAG}: action=end"
